import numpy as np
import sys
import matplotlib.pyplot as plt
import util.file_handler as file_handler
import util.classifiers as classifiers
import constants
import pysa.utils as utils
import util.plotter as plotter


def start(username='', el=0, baseline_length=250, desync_length=1500, ratio_trained=0.84, imf_index=0):

    # -------------------------------------------USER INFORMATION AND PARAMETERS-------------------------------------- #
    if username == '':
        print "Please specify subject username in 'main.py'"
        sys.exit()
    trial_length = 2250
    stimuli_onset = 250
    sample_frequency = 500.0
    frequency_label = 'frequency'
    amplitude_label = 'amplitude'
    # ---------------------------------------------------------------------------------------------------------------- #

    # -------------------------------------------FILENAMES------------------------------------------------------------ #
    subfolder = 'NHHT/'
    filename_seg = username + '_data_motor_cortex_segmented.mat'
    filename_bad_trials_matrix = 'bad_trials_matrix.mat'
    fn_left = subfolder + 'nht_left_hand.mat'
    fn_right = subfolder + 'nht_right_hand.mat'
    fn_neutral = subfolder + 'nht_neutral.mat'

    # ---------------------------------------------------------------------------------------------------------------- #
    data_json = file_handler.read_mat_file_into_json_object(username, filename_seg)
    events = data_json['events']

    # ------------------------------------------BAD TRIALS MATRIX----------------------------------------------------- #
    data_json = file_handler.read_mat_file_into_json_object(username, filename_bad_trials_matrix)
    bad_trials_matrix = data_json[filename_bad_trials_matrix[:-4]]
    # ---------------------------------------------------------------------------------------------------------------- #

    # --------------------------------------------LOAD .mat FILES INTO JSON------------------------------------------- #
    left_json = file_handler.read_mat_file_into_json_object(username, fn_left)
    right_json = file_handler.read_mat_file_into_json_object(username, fn_right)
    neutral_json = file_handler.read_mat_file_into_json_object(username, fn_neutral)

    # ---------------------------------------------------------------------------------------------------------------- #

    # --------------------------------------------FREQUENCIES AND AMPLITUDES------------------------------------------ #
    left_frequency_matrix = left_json[frequency_label]
    left_amplitude_matrix = left_json[amplitude_label]

    right_frequency_matrix = right_json[frequency_label]
    right_amplitude_matrix = right_json[amplitude_label]

    neutral_frequency_matrix = neutral_json[frequency_label]
    neutral_amplitude_matrix = neutral_json[amplitude_label]

    # ---------------------------------------------------------------------------------------------------------------- #
    # ---------------------------------------------------------------------------------------------------------------- #
    # ---------------------------------------------------------------------------------------------------------------- #

    # ---------------------------------------------------------------------------------------------------------------- #
    #                                                DATA ANALYSIS                                                     #
    # ---------------------------------------------------------------------------------------------------------------- #
    # print("\x1b[2J\x1b[H")
    introduction = "EEG classification scheme - Subject = " + username.title()
    print "%s\n%s\n%s" % ('-' * len(introduction), introduction, '-' * len(introduction))

    n_electrodes, n_imfs, left_data_matrix_length = np.shape(left_frequency_matrix)
    n_trials_left = int(left_data_matrix_length/trial_length)
    n_trials_right = int(np.shape(right_frequency_matrix)[-1]/trial_length)
    n_trials_neutral = int(np.shape(neutral_frequency_matrix)[-1]/trial_length)

    bad_trials_left = np.zeros((n_electrodes, n_trials_left), np.int)
    bad_trials_right = np.zeros((n_electrodes, n_trials_right), np.int)
    bad_trials_neutral = np.zeros((n_electrodes, n_trials_neutral), np.int)

    for elec in range(len(bad_trials_matrix)):
        l_counter = 0
        r_counter = 0
        n_counter = 0
        for trial in range(len(bad_trials_matrix[elec])):

            if events[trial] == 'L':
                if bad_trials_matrix[elec, trial]:
                    bad_trials_left[elec, l_counter] = 1
                l_counter += 1

            elif events[trial] == 'R':
                if bad_trials_matrix[elec, trial]:
                    bad_trials_right[elec, r_counter] = 1
                r_counter += 1

            elif events[trial] == 'N':
                if bad_trials_matrix[elec, trial]:
                    bad_trials_neutral[elec, n_counter] = 1
                n_counter += 1

    left_vs_neutral = False
    right_vs_neutral = False
    left_vs_right = True

    plotting = False
    plot_features = False

    la = left_amplitude_matrix[el, imf_index, :]
    ra = right_amplitude_matrix[el, imf_index, :]
    na = neutral_amplitude_matrix[el, imf_index, :]

    bad_left = bad_trials_left[el, :]
    bad_right = bad_trials_right[el, :]
    bad_neutral = bad_trials_neutral[el, :]

    features_left = get_erds_feature_vector(la, bad_left, n_trials_left, stimuli_onset, desync_length, trial_length,
                                            baseline_length=baseline_length)
    features_right = get_erds_feature_vector(ra, bad_right, n_trials_right, stimuli_onset, desync_length, trial_length,
                                             baseline_length=baseline_length)
    features_neutral = get_erds_feature_vector(na, bad_neutral, n_trials_neutral, stimuli_onset, desync_length,
                                               trial_length, baseline_length=baseline_length)

    if plot_features:
        if left_vs_neutral:
            plot_feature_vector(features_left, 'left')
            # plot_feature_vector(features_right, 'right')
            plot_feature_vector(features_neutral, 'neutral')

    n_trials_left = len(features_left)
    n_trials_right = len(features_right)
    n_trials_neutral = len(features_neutral)

    print 'Accepted trials Left: ' + str(n_trials_left)
    print 'Accepted trials Right: ' + str(n_trials_right)
    print 'Accepted trials Neutral: ' + str(n_trials_neutral)
    print 'Ratio trained: ' + str(ratio_trained)
    n_train_left = int(n_trials_left*ratio_trained)
    n_train_right = int(n_trials_right*ratio_trained)
    n_train_neutral = int(n_trials_neutral*ratio_trained)

    n_predict_left = n_trials_left - n_train_left
    n_predict_right = n_trials_right - n_train_right
    n_predict_neutral = n_trials_neutral - n_train_neutral

    if left_vs_neutral:
        n_train_total = n_train_left + n_train_neutral
        n_pred_total = n_predict_left + n_predict_neutral
        classes_training = np.zeros(n_train_total, np.int)
        classes_pred = np.zeros(n_pred_total, np.int)

        neutral_trained = features_neutral[:n_train_neutral]
        neutral_predict = features_neutral[n_train_neutral:]
        left_trained = features_left[:n_train_left]
        left_predict = features_left[n_train_left:]
        training_set = np.vstack((neutral_trained, left_trained))

        prediction_set = np.vstack((neutral_predict, left_predict))
        classes_pred[n_predict_neutral:] += 1

        classes_training[n_train_neutral:] += 1

    elif right_vs_neutral:
        n_train_total = n_train_right + n_train_neutral
        n_pred_total = n_predict_right + n_predict_neutral

        classes_training = np.zeros(n_train_total, np.int)
        classes_pred = np.zeros(n_pred_total, np.int)

        neutral_trained = features_neutral[:n_train_neutral]
        neutral_predict = features_neutral[n_train_neutral:]
        right_trained = features_right[:n_train_right]
        right_predict = features_right[n_train_right:]
        training_set = np.vstack((neutral_trained, right_trained))
        prediction_set = np.vstack((neutral_predict, right_predict))

        classes_training[n_train_neutral:] += 1
        classes_pred[n_predict_neutral:] += 1

    elif left_vs_right:
        n_train_total = n_train_right + n_train_left
        n_pred_total = n_predict_right + n_predict_left

        classes_training = np.zeros(n_train_total, np.int)
        classes_pred = np.zeros(n_pred_total, np.int)

        right_trained = features_right[:n_train_right]
        right_predict = features_right[n_train_right:]
        left_trained = features_left[:n_train_left]
        left_predict = features_left[n_train_left:]

        training_set = np.vstack((right_trained, left_trained))
        prediction_set = np.vstack((right_predict, left_predict))

        classes_training[:n_train_right] += 1
        classes_pred[:n_predict_right] += 1

    print np.shape(training_set)
    print np.shape(prediction_set)

    classifier_name = 'rbf'
    gamma_key = 'gamma'
    c_key = 'C'

    params = {
        c_key: [1, 2, 4, 6, 8, 10],
        gamma_key: [0.0001, 0.001, 0.01, 0.1, 1.0, 2.0]
    }

    classifier, optimal_params = classifiers.grid_search(classifier_name, (training_set, classes_training), params)

    # print optimal_params
    score = classifiers.predict_test_data(classifier, (prediction_set, classes_pred))

    if plotting:
        if left_vs_neutral:
            la_avg = compute_average_amplitude(left_amplitude_matrix[el, imf_index], bad_trials_left, trial_length)
            na_avg = compute_average_amplitude(neutral_amplitude_matrix[el, imf_index], bad_trials_neutral, trial_length)
            # la = left_amplitude_matrix[el, imf_index, 6*trial_length:7*trial_length]
            # na = neutral_amplitude_matrix[el, imf_index, 7*trial_length:8*trial_length]
            plotter.plot_erds(la_avg, 'Left')
            plotter.plot_erds(na_avg, 'Neutral')
            # plotter.plot_erds(la, 'Left single')
            # plotter.plot_erds(na, 'Neutral single')
        elif right_vs_neutral:
            ra_avg = compute_average_amplitude(right_amplitude_matrix[el, imf_index], bad_trials_right, trial_length)
            na_avg = compute_average_amplitude(neutral_amplitude_matrix[el, imf_index], bad_trials_neutral, trial_length)
            # ra = right_amplitude_matrix[el, imf_index, 6*trial_length:7*trial_length]
            # na = neutral_amplitude_matrix[el, imf_index, 7*trial_length:8*trial_length]
            plotter.plot_erds(ra_avg, 'Right')
            plotter.plot_erds(na_avg, 'Neutral')
            # plotter.plot_erds(ra, 'Right single')
            # plotter.plot_erds(na, 'Neutral single')
        elif left_vs_right:
            la_avg = compute_average_amplitude(left_amplitude_matrix[el, imf_index], bad_trials_left, trial_length)
            ra_avg = compute_average_amplitude(right_amplitude_matrix[el, imf_index], bad_trials_right, trial_length)
            plotter.plot_erds(la_avg, 'Left')
            plotter.plot_erds(ra_avg, 'Right')

    if plotting or plot_features:
        try:
            plt.draw()
            plt.show(block=True)
        except KeyboardInterrupt, ki:
            sys.exit()

    return score


def get_erds_feature_vector(data_array, bad_trials, n_trials=70, stimulus_onset=250, desync_length=1500, trial_length=2250,
                            baseline_length=200):

    features = np.ndarray(([]), np.float)
    first_trial = True
    trial_num = 0

    for k in range(0, n_trials*trial_length, trial_length):

        if not bad_trials[trial_num]:

            start_sample = k
            end_sample = start_sample + stimulus_onset + desync_length
            data = data_array[start_sample:end_sample]
            min_data = min(data)
            max_data = max(data)
            n_samples = len(data)
            power = utils.normalize_data(data, min_data, max_data, n_samples)**2

            mean_baseline_power = float(np.mean(power[:baseline_length]))
            erds = (100.0*(power-mean_baseline_power))/mean_baseline_power
            mean_erds = np.mean(erds[baseline_length:])

            if first_trial:
                first_trial = False
                features = mean_erds
            else:
                features = np.vstack((features, mean_erds))

        trial_num += 1
    return features


def compute_average_amplitude(data_matrix, bad_trials_matrix, trial_length=2250):
    dim = 1
    if len(np.shape(data_matrix)) == 3:
        dim = 3
        n_electrodes, n_imfs, data_length = np.shape(data_matrix)

    elif len(np.shape(data_matrix)) == 2:
        dim = 2
        n_imfs, data_length = np.shape(data_matrix)
        n_electrodes = 1

    else:
        n_electrodes = 1
        n_imfs = 1
        data_length = len(data_matrix)

    avg_data = np.zeros((n_electrodes, n_imfs, trial_length), np.float)

    for el in range(n_electrodes):
        counter = 0
        for trial in range(0, data_length, trial_length):
            # Check if current trial (at current electrode) is an accepted trial:
            if not bad_trials_matrix[el, int(trial/trial_length)]:
                counter += 1
                if dim == 3:
                    data = data_matrix[el, :, trial:trial+trial_length]
                    for imf in range(n_imfs):
                        avg_data[el, imf, :] += (data[imf] - np.mean(data[imf]))

                elif dim == 2:
                    data = data_matrix[:, trial:trial+trial_length]
                    for imf in range(n_imfs):
                        avg_data[el, imf, :] += (data[imf] - np.mean(data[imf]))
                else:
                    data = data_matrix[trial:trial+trial_length]
                    avg_data[el, 0, :] += (data - np.mean(data))

        if counter > 1:

            avg_data[el, :, :] /= counter
        else:
            avg_data[el, :, :] = data_matrix[el, :, trial:trial+trial_length]

    return avg_data


def plot_feature_vector(feature_vector, title=''):

    fig = plt.figure()
    plt.subplots_adjust(left=0.10, bottom=0.05, right=0.98, top=0.92)
    ax = fig.add_subplot(111, axisbg='lightgoldenrodyellow')
    ax.autoscale(True)
    ax.set_title(title)
    ax.set_xlabel('Trial #')
    ax.set_ylabel('Mean ERD/ERS')
    ax.plot(feature_vector, color='b', linewidth=2.0, label=title)
    ax.plot(np.ones(len(feature_vector))*np.mean(feature_vector), color='r', linewidth=1.5, label='Mean')

    legend = ax.legend(loc='upper left', shadow=True)
    # The frame is matplotlib.patches.Rectangle instance surrounding the legend.
    legend_frame = legend.get_frame()
    legend_frame.set_facecolor('0.90')

    # Set the fontsize
    for label in legend.get_texts():
        label.set_fontsize('large')
    for label in legend.get_lines():
        label.set_linewidth(1.5)  # the legend line width



















