import numpy as np
import sys
import matplotlib.pyplot as plt
import util.file_handler as file_handler
import util.classifiers_no_print as classifiers
import constants
import pysa.utils as utils


def start(username='', el=0, baseline_length=250, desync_length=1500, ratio_trained=0.84, imf_index=0):

    # -------------------------------------------USER INFORMATION AND PARAMETERS-------------------------------------- #
    if username == '':
        print "Please specify subject username in 'main.py'"
        sys.exit()
    trial_length = 2250
    stimuli_onset = 250
    sample_frequency = 500.0
    frequency_label = 'frequency'
    amplitude_label = 'amplitude'
    # ---------------------------------------------------------------------------------------------------------------- #

    # -------------------------------------------FILENAMES------------------------------------------------------------ #
    subfolder = 'NHHT/'
    filename_seg = username + '_data_motor_cortex_segmented.mat'
    filename_bad_trials_matrix = 'bad_trials_matrix.mat'
    fn_left = subfolder + 'nht_left_hand.mat'
    fn_right = subfolder + 'nht_right_hand.mat'
    fn_neutral = subfolder + 'nht_neutral.mat'

    # ---------------------------------------------------------------------------------------------------------------- #
    data_json = file_handler.read_mat_file_into_json_object(username, filename_seg)
    events = data_json['events']

    # ------------------------------------------BAD TRIALS MATRIX----------------------------------------------------- #
    data_json = file_handler.read_mat_file_into_json_object(username, filename_bad_trials_matrix)
    bad_trials_matrix = data_json[filename_bad_trials_matrix[:-4]]
    # ---------------------------------------------------------------------------------------------------------------- #

    # --------------------------------------------LOAD .mat FILES INTO JSON------------------------------------------- #
    left_json = file_handler.read_mat_file_into_json_object(username, fn_left)
    right_json = file_handler.read_mat_file_into_json_object(username, fn_right)
    neutral_json = file_handler.read_mat_file_into_json_object(username, fn_neutral)

    # ---------------------------------------------------------------------------------------------------------------- #

    # --------------------------------------------FREQUENCIES AND AMPLITUDES------------------------------------------ #
    left_frequency_matrix = left_json[frequency_label]
    left_amplitude_matrix = left_json[amplitude_label]

    right_frequency_matrix = right_json[frequency_label]
    right_amplitude_matrix = right_json[amplitude_label]

    neutral_frequency_matrix = neutral_json[frequency_label]
    neutral_amplitude_matrix = neutral_json[amplitude_label]

    # ---------------------------------------------------------------------------------------------------------------- #
    # ---------------------------------------------------------------------------------------------------------------- #
    # ---------------------------------------------------------------------------------------------------------------- #

    # ---------------------------------------------------------------------------------------------------------------- #
    #                                                DATA ANALYSIS                                                     #
    # ---------------------------------------------------------------------------------------------------------------- #

    n_electrodes, n_imfs, left_data_matrix_length = np.shape(left_frequency_matrix)
    n_trials_left = int(left_data_matrix_length/trial_length)
    n_trials_right = int(np.shape(right_frequency_matrix)[-1]/trial_length)
    n_trials_neutral = int(np.shape(neutral_frequency_matrix)[-1]/trial_length)

    bad_trials_left = np.zeros((n_electrodes, n_trials_left), np.int)
    bad_trials_right = np.zeros((n_electrodes, n_trials_right), np.int)
    bad_trials_neutral = np.zeros((n_electrodes, n_trials_neutral), np.int)

    for elec in range(len(bad_trials_matrix)):
        l_counter = 0
        r_counter = 0
        n_counter = 0
        for trial in range(len(bad_trials_matrix[elec])):

            if events[trial] == 'L':
                if bad_trials_matrix[elec, trial]:
                    bad_trials_left[elec, l_counter] = 1
                l_counter += 1

            elif events[trial] == 'R':
                if bad_trials_matrix[elec, trial]:
                    bad_trials_right[elec, r_counter] = 1
                r_counter += 1

            elif events[trial] == 'N':
                if bad_trials_matrix[elec, trial]:
                    bad_trials_neutral[elec, n_counter] = 1
                n_counter += 1

    left_vs_neutral = False
    right_vs_neutral = False
    left_vs_right = True

    # Important parameters for classification
    # el = constants.electrodes_motor_cortex.index(constants.cp1[0])

    la = left_amplitude_matrix[el, imf_index, :]
    ra = right_amplitude_matrix[el, imf_index, :]
    na = neutral_amplitude_matrix[el, imf_index, :]

    bad_left = bad_trials_left[el, :]
    bad_right = bad_trials_right[el, :]
    bad_neutral = bad_trials_neutral[el, :]

    features_left = get_erds_feature_vector(la, bad_left, n_trials_left, stimuli_onset, desync_length, trial_length,
                                            baseline_length=baseline_length)
    features_right = get_erds_feature_vector(ra, bad_right, n_trials_right, stimuli_onset, desync_length, trial_length,
                                             baseline_length=baseline_length)
    features_neutral = get_erds_feature_vector(na, bad_neutral, n_trials_neutral, stimuli_onset, desync_length,
                                               trial_length, baseline_length=baseline_length)

    n_trials_left = len(features_left)
    n_trials_right = len(features_right)
    n_trials_neutral = len(features_neutral)

    n_train_left = int(n_trials_left*ratio_trained)
    n_train_right = int(n_trials_right*ratio_trained)
    n_train_neutral = int(n_trials_neutral*ratio_trained)

    n_predict_left = n_trials_left - n_train_left
    n_predict_right = n_trials_right - n_train_right
    n_predict_neutral = n_trials_neutral - n_train_neutral

    if left_vs_neutral:
        n_train_total = n_train_left + n_train_neutral
        n_pred_total = n_predict_left + n_predict_neutral
        classes_training = np.zeros(n_train_total, np.int)
        classes_pred = np.zeros(n_pred_total, np.int)

        neutral_trained = features_neutral[:n_train_neutral]
        neutral_predict = features_neutral[n_train_neutral:]
        left_trained = features_left[:n_train_left]
        left_predict = features_left[n_train_left:]
        training_set = np.vstack((neutral_trained, left_trained))

        prediction_set = np.vstack((neutral_predict, left_predict))
        classes_pred[n_predict_neutral:] += 1

        classes_training[n_train_neutral:] += 1

    elif right_vs_neutral:
        n_train_total = n_train_right + n_train_neutral
        n_pred_total = n_predict_right + n_predict_neutral

        classes_training = np.zeros(n_train_total, np.int)
        classes_pred = np.zeros(n_pred_total, np.int)

        neutral_trained = features_neutral[:n_train_neutral]
        neutral_predict = features_neutral[n_train_neutral:]
        right_trained = features_right[:n_train_right]
        right_predict = features_right[n_train_right:]
        training_set = np.vstack((neutral_trained, right_trained))
        prediction_set = np.vstack((neutral_predict, right_predict))

        classes_training[n_train_neutral:] += 1
        classes_pred[n_predict_neutral:] += 1

    elif left_vs_right:
        n_train_total = n_train_right + n_train_left
        n_pred_total = n_predict_right + n_predict_left

        classes_training = np.zeros(n_train_total, np.int)
        classes_pred = np.zeros(n_pred_total, np.int)

        right_trained = features_right[:n_train_right]
        right_predict = features_right[n_train_right:]
        left_trained = features_left[:n_train_left]
        left_predict = features_left[n_train_left:]

        training_set = np.vstack((right_trained, left_trained))
        prediction_set = np.vstack((right_predict, left_predict))

        classes_training[:n_train_right] += 1
        classes_pred[:n_predict_right] += 1

    classifier_name = 'rbf'
    gamma_key = 'gamma'
    c_key = 'C'

    params = {
        c_key: [1, 2, 4, 6, 8, 10],
        gamma_key: [0.0001, 0.001, 0.01, 0.1, 1.0, 2.0]
    }

    classifier, optimal_params = classifiers.grid_search(classifier_name, (training_set, classes_training), params)

    # print optimal_params
    score = classifiers.predict_test_data(classifier, (prediction_set, classes_pred))

    return score


def get_erds_feature_vector(data_array, bad_trials, n_trials=70, stimulus_onset=250, desync_length=1500, trial_length=2250,
                            baseline_length=200):

    features = np.ndarray(([]), np.float)
    first_trial = True
    trial_num = 0

    for k in range(0, n_trials*trial_length, trial_length):

        if not bad_trials[trial_num]:

            start_sample = k
            end_sample = start_sample + stimulus_onset + desync_length
            data = data_array[start_sample:end_sample]
            min_data = min(data)
            max_data = max(data)
            n_samples = len(data)
            power = utils.normalize_data(data, min_data, max_data, n_samples)**2

            mean_baseline_power = float(np.mean(power[:baseline_length]))
            erds = (100.0*(power-mean_baseline_power))/mean_baseline_power
            # for i in range(len(erds)):
            #     if erds[i] > 100.0:
            #         erds[i] = 100.0
            #     elif erds[i] < -100.0:
            #         erds[i] = -100.0

            mean_erds = np.mean(erds[baseline_length:])

            if first_trial:
                first_trial = False
                features = mean_erds
            else:
                features = np.vstack((features, mean_erds))

        trial_num += 1
    return features








