import numpy as np
import sys
import matplotlib.pyplot as plt
import util.file_handler as file_handler
import util.plotter as plotter
import util.plotter_scroll as plotter_scroll
import constants
import pysa.utils as utils


def start(username=''):

    if username == '':
        print "Please specify subject username in 'main.py'"
        sys.exit()

    # -------------------------------------------USER INFORMATION AND PARAMETERS-------------------------------------- #
    trial_length = 2250
    stimuli_onset = 250
    sample_frequency = 500.0
    frequency_label = 'frequency'
    amplitude_label = 'amplitude'
    # ---------------------------------------------------------------------------------------------------------------- #

    # -------------------------------------------FILENAMES------------------------------------------------------------ #
    subfolder = 'NHHT/'
    fn_left = subfolder + 'nht_left_hand.mat'
    fn_right = subfolder + 'nht_right_hand.mat'
    fn_neutral = subfolder + 'nht_neutral.mat'
    filename_bad_trials_matrix = 'bad_trials_matrix.mat'
    filename_seg = username + '_data_motor_cortex_segmented.mat'

    # ---------------------------------------------------------------------------------------------------------------- #
    data_json = file_handler.read_mat_file_into_json_object(username, filename_seg)
    events = data_json['events']
    # ---------------------------------------------------------------------------------------------------------------- #

    # --------------------------------------------LOAD .mat FILES INTO JSON------------------------------------------- #
    left_json = file_handler.read_mat_file_into_json_object(username, fn_left)
    right_json = file_handler.read_mat_file_into_json_object(username, fn_right)
    neutral_json = file_handler.read_mat_file_into_json_object(username, fn_neutral)

    # ---------------------------------------------------------------------------------------------------------------- #

    # ------------------------------------------BAD TRIALS DATA------------------------------------------------------- #
    data_json = file_handler.read_mat_file_into_json_object(username, filename_bad_trials_matrix)
    bad_trials_matrix = data_json[filename_bad_trials_matrix[:-4]]
    # ---------------------------------------------------------------------------------------------------------------- #

    # --------------------------------------------FREQUENCIES AND AMPLITUDES------------------------------------------ #
    left_frequency_matrix = left_json[frequency_label]
    left_amplitude_matrix = left_json[amplitude_label]

    right_frequency_matrix = right_json[frequency_label]
    right_amplitude_matrix = right_json[amplitude_label]

    neutral_frequency_matrix = neutral_json[frequency_label]
    neutral_amplitude_matrix = neutral_json[amplitude_label]

    # ---------------------------------------------------------------------------------------------------------------- #
    # ---------------------------------------------------------------------------------------------------------------- #
    # ---------------------------------------------------------------------------------------------------------------- #

    # ---------------------------------------------------------------------------------------------------------------- #
    #                                                DATA ANALYSIS                                                     #
    # ---------------------------------------------------------------------------------------------------------------- #
    n_electrodes, n_imfs, left_data_matrix_length = np.shape(left_frequency_matrix)

    n_trials_left = int(left_data_matrix_length/trial_length)
    n_trials_right = int(np.shape(right_frequency_matrix)[-1]/trial_length)
    n_trials_neutral = int(np.shape(neutral_frequency_matrix)[-1]/trial_length)

    bad_trials_left = np.zeros((n_electrodes, n_trials_left), np.int)
    bad_trials_right = np.zeros((n_electrodes, n_trials_right), np.int)
    bad_trials_neutral = np.zeros((n_electrodes, n_trials_neutral), np.int)

    for el in range(len(bad_trials_matrix)):
        l_counter = 0
        r_counter = 0
        n_counter = 0
        for trial in range(len(bad_trials_matrix[el])):

            if events[trial] == 'L':
                if bad_trials_matrix[el, trial]:
                    bad_trials_left[el, l_counter] = 1
                l_counter += 1

            elif events[trial] == 'R':
                if bad_trials_matrix[el, trial]:
                    bad_trials_right[el, r_counter] = 1
                r_counter += 1

            elif events[trial] == 'N':
                if bad_trials_matrix[el, trial]:
                    bad_trials_neutral[el, n_counter] = 1
                n_counter += 1

    left_frequency_avg = compute_average_frequency(left_frequency_matrix, bad_trials_left, n_trials_left, trial_length)
    left_amplitude_avg = compute_average_amplitude(left_amplitude_matrix, bad_trials_left, n_trials_left, trial_length)

    right_frequency_avg = compute_average_frequency(right_frequency_matrix, bad_trials_right, n_trials_right, trial_length)
    right_amplitude_avg = compute_average_amplitude(right_amplitude_matrix, bad_trials_right, n_trials_right, trial_length)

    neutral_frequency_avg = compute_average_frequency(neutral_frequency_matrix, bad_trials_neutral, n_trials_neutral,
                                                  trial_length)
    neutral_amplitude_avg = compute_average_amplitude(neutral_amplitude_matrix, bad_trials_neutral, n_trials_neutral,
                                                  trial_length)

    el = constants.electrodes_motor_cortex.index(constants.c2[0])
    el_str = 'C2'
    imf_start = 1
    imf_end = 2
    end_sample = 2000
    time_axis = np.linspace(0, end_sample-1, end_sample)

    lf = left_frequency_avg[el, imf_start:imf_end, :end_sample]
    la = left_amplitude_avg[el, imf_start:imf_end, :end_sample]

    rf = right_frequency_avg[el, imf_start:imf_end, :end_sample]
    ra = right_amplitude_avg[el, imf_start:imf_end, :end_sample]

    nf = neutral_frequency_avg[el, imf_start:imf_end, :end_sample]
    na = neutral_amplitude_avg[el, imf_start:imf_end, :end_sample]

    plotter.plot_erds(la, electrode=el_str, event='Stm: Left')
    plotter.plot_erds(ra, electrode=el_str, event='Stm: Right')
    # plotter.plot_erds(na, electrode=el_str, event='Stm: Neutral')

    # start_trial = 32*trial_length
    # end_trial = start_trial + end_sample

    # lfs = left_frequency_matrix[el, imf_start:imf_end, start_trial:end_trial]
    # las = left_amplitude_matrix[el, imf_start:imf_end, start_trial:end_trial]
    # #
    # # rfs = right_frequency_matrix[el, imf_start:imf_end, start_trial:end_trial]
    # # ras = right_amplitude_matrix[el, imf_start:imf_end, start_trial:end_trial]
    # #
    # nfs = neutral_frequency_matrix[el, imf_start:imf_end, start_trial:end_trial]
    # nas = neutral_amplitude_matrix[el, imf_start:imf_end, start_trial:end_trial]
    # #
    # plotter.plot_erds(las, electrode=el_str, event='Stm: Left')
    # plotter.plot_erds(ras, electrode=el_str, event='R single')
    # plotter.plot_erds(nas, electrode=el_str, event='Stm: Neutral')
    # #
    # # plotter.plot_single_channel(la[0], plt, 'neutral imf 1')
    #
    plotter.plot_hilbert_spectra_multiple_imfs(time_axis, nf, na, el_str, plt,
                                               sample_frequency, events=['Left'], stimulus_onset_in_ms=500,
                                               baseline_correct=True)

    plotter.plot_hilbert_spectra_multiple_imfs(time_axis, rf, ra, el_str, plt,
                                               sample_frequency, events=['Right'], stimulus_onset_in_ms=500, baseline_correct=True)

    # plotter.plot_hilbert_spectra_multiple_imfs(time_axis, nf, na, el_str, plt, sample_frequency,
    #                                            events=['Neutral'], stimulus_onset_in_ms=500, baseline_correct=True)

    # SINGLE TRIAL:

    # plotter.plot_hilbert_spectra_multiple_imfs(time_axis, lfs, las, el_str, plt,
    #                                            sample_frequency, events='L', stimulus_onset_in_ms=500,
    #                                            baseline_correct=True)

    # plotter.plot_hilbert_spectra_multiple_imfs(time_axis, rfs, ras, el_str, plt,
    #                                            sample_frequency, events='R', stimulus_onset_in_ms=500, baseline_correct=True)

    # plotter.plot_hilbert_spectra_multiple_imfs(time_axis, nfs, nas, el_str, plt, sample_frequency,
    #                                            events='N', stimulus_onset_in_ms=500, baseline_correct=True)

    try:
        plt.draw()
        plt.show(block=True)
    except KeyboardInterrupt, ki:
        sys.exit()


def compute_average_amplitude(data_matrix, bad_trials_matrix, n_trials, trial_length=2250):

    if len(np.shape(data_matrix)) == 3:
        n_electrodes, n_imfs, data_length = np.shape(data_matrix)
    else:
        n_imfs, data_length = np.shape(data_matrix)
        n_electrodes = 1
    avg_data = np.zeros((n_electrodes, n_imfs, trial_length), np.float)

    for el in range(n_electrodes):
        counter = 0
        for trial in range(0, data_length, trial_length):
            # Check if current trial (at current electrode) is an accepted trial:
            if not bad_trials_matrix[el, int(trial/trial_length)]:
                counter += 1
                data = data_matrix[el, :, trial:trial+trial_length]

                for imf in range(len(data)):
                    avg_data[el, imf, :] += (data[imf] - np.mean(data[imf]))
                    # avg_data[el, imf, :] += data[imf]
        if counter > 1:
            avg_data[el, :, :] /= counter
        else:
            avg_data[el, :, :] = data_matrix[el, :, trial:trial+trial_length]

    return avg_data


def compute_average_frequency(data_matrix, bad_trials_matrix, n_trials, trial_length=2250):

    if len(np.shape(data_matrix)) == 3:
        n_electrodes, n_imfs, data_length = np.shape(data_matrix)
    else:
        n_imfs, data_length = np.shape(data_matrix)
        n_electrodes = 1
    avg_data = np.zeros((n_electrodes, n_imfs, trial_length), np.float)

    for el in range(n_electrodes):
        counter = 0
        for trial in range(0, data_length, trial_length):
            if not bad_trials_matrix[el, int(trial/trial_length)]:
                data = data_matrix[el, :, trial:trial+trial_length]
                avg_data[el, :, :] += data
                counter += 1
        if counter > 1:
            avg_data[el, :, :] /= counter

    return avg_data


def compute_average_erd(amplitude_matrix, bad_trials_matrix, trial_length=2250, stimulus_onset=250):

    n_imfs, data_length = np.shape(amplitude_matrix)
    avg_data = np.zeros((n_imfs, trial_length), np.float)
    counter = 0
    for trial in range(0, data_length, trial_length):
        # Check if current trial (at current electrode) is an accepted trial:
        trial_num = int(trial/trial_length)
        if not bad_trials_matrix[trial_num]:
            counter += 1
            data = amplitude_matrix[:, trial:trial+trial_length]

            for imf in range(len(data)):
                min_data = min(data[imf])
                max_data = max(data[imf])
                power = utils.normalize_data(data[imf], min_data, max_data, trial_length)**2

                baseline_length = stimulus_onset - 50
                mean_baseline_power = float(np.mean(power[:baseline_length]))
                erd = (100.0*(power - mean_baseline_power))/mean_baseline_power
                for k in range(len(erd)):
                    if erd[k] > 100.0:
                        erd[k] = 100.0
                    elif erd[k] < -100.0:
                        erd[k] = -100.0
                avg_data[imf, :] += erd

    if counter >= 1:
        print counter
        avg_data /= counter
    else:
        print "No accepted trials! Terminating program..."
        sys.exit()

    return avg_data
