import sys
import matplotlib.pyplot as plt
import util.file_handler as file_handler
import util.plotter_scroll as plotter_scroll
import numpy as np
import util.filters as filt
import util.plotter as plotter
import pysa.emd as emd
import pysa.utils as utils
def start(username=''):

    # --------------------------------------USER INFORMATION AND PARAMETERS------------------------------------------- #
    if username == '':
        print "Please specify subject username in 'main.py'"
        sys.exit()

    sample_frequency = 500.0
    samples_per_trial = 3000
    frequency_label = 'frequency'
    amplitude_label = 'amplitude'
    # ---------------------------------------------------------------------------------------------------------------- #

    # -------------------------------------------FILENAMES------------------------------------------------------------ #
    filename_seg = username + '_data_motor_cortex_segmented.mat'
    filename_seg_all = username + '_data_256_segmented.mat'
    filename_filtered_data = 'data_matrix_emd_filtered.mat'
    filename_line_noise_matrix = 'line_noise_matrix.mat'
    filename_trend_matrix = 'trend_matrix.mat'
    filename_bad_trials_matrix = 'bad_trials_matrix.mat'
    filename_cleaned_data = 'data_matrix_cleaned.mat'
    filename_low_frequency_matrix = 'low_frequency_matrix.mat'

    subfolder = 'NHHT/'
    fn_left = subfolder + 'nht_left_hand.mat'
    fn_right = subfolder + 'nht_right_hand.mat'
    fn_neutral = subfolder + 'nht_neutral.mat'

    # --------------------------------------------LOAD .mat FILES INTO JSON------------------------------------------- #
    left_json = file_handler.read_mat_file_into_json_object(username, fn_left)
    right_json = file_handler.read_mat_file_into_json_object(username, fn_right)
    neutral_json = file_handler.read_mat_file_into_json_object(username, fn_neutral)

    # ---------------------------------------------------------------------------------------------------------------- #

    # --------------------------------------------FREQUENCIES AND AMPLITUDES------------------------------------------ #
    left_frequency_matrix = left_json[frequency_label]
    left_amplitude_matrix = left_json[amplitude_label]

    right_frequency_matrix = right_json[frequency_label]
    right_amplitude_matrix = right_json[amplitude_label]

    neutral_frequency_matrix = neutral_json[frequency_label]
    neutral_amplitude_matrix = neutral_json[amplitude_label]

    # ---------------------------------------------------------------------------------------------------------------- #

    # ---------------------------------------------------------------------------------------------------------------- #

    # ------------------------------------------RAW EEG DATA---------------------------------------------------------- #
    data_json = file_handler.read_mat_file_into_json_object(username, filename_seg)
    data_matrix_raw = data_json['data_seg']
    events = data_json['events']
    # ---------------------------------------------------------------------------------------------------------------- #

    # ------------------------------------------EMD-FILTERED DATA----------------------------------------------------- #
    data_json = file_handler.read_mat_file_into_json_object(username, filename_filtered_data)
    data_matrix_filtered = data_json[filename_filtered_data[:-4]]
    # ---------------------------------------------------------------------------------------------------------------- #

    # ------------------------------------------TREND-MATRIX and LINE NOISE DATA-------------------------------------- #
    data_json = file_handler.read_mat_file_into_json_object(username, filename_trend_matrix)
    trend_matrix = data_json[filename_trend_matrix[:-4]]

    data_json = file_handler.read_mat_file_into_json_object(username, filename_line_noise_matrix)
    line_noise_matrix = data_json[filename_line_noise_matrix[:-4]]
    # ---------------------------------------------------------------------------------------------------------------- #

    # ------------------------------------------CLEANED DATA---------------------------------------------------------- #
    data_json = file_handler.read_mat_file_into_json_object(username, filename_cleaned_data)
    data_matrix_clean = data_json[filename_cleaned_data[:-4]]
    # ---------------------------------------------------------------------------------------------------------------- #
    # ------------------------------------------LOW-FREQUENCY DATA---------------------------------------------------- #
    data_json = file_handler.read_mat_file_into_json_object(username, filename_low_frequency_matrix)
    low_freq_matrix = data_json[filename_low_frequency_matrix[:-4]]
    # ---------------------------------------------------------------------------------------------------------------- #

    # ------------------------------------------BAD TRIALS------------------------------------------------------------ #
    data_json = file_handler.read_mat_file_into_json_object(username, filename_bad_trials_matrix)
    bad_trials_matrix = data_json[filename_bad_trials_matrix[:-4]]
    # ---------------------------------------------------------------------------------------------------------------- #

    # ---------------------------------------------------------------------------------------------------------------- #
    # ---------------------------------------------------------------------------------------------------------------- #
    # ---------------------------------------------------------------------------------------------------------------- #

    # ---------------------------------------------------------------------------------------------------------------- #
    #                                                DATA PLOTTING                                                     #
    # ---------------------------------------------------------------------------------------------------------------- #
    print("\x1b[2J\x1b[H")
    introduction = "EEG visualization session - Subject = " + username.title()
    print "%s\n%s\n%s" % ('-' * len(introduction), introduction, '-' * len(introduction))

    # Raw EEG data:
    plotter_scroll.plot_data_sliding(data_matrix_raw, events, sample_frequency, offset=50.0, title='Raw data',
                                     n_sec_per_trial=6, n_trial_per_frame=1)

    # EMD-filtered EEG data:
    plotter_scroll.plot_data_sliding(data_matrix_filtered, events, sample_frequency, title='EMD-filtered data',
                                     n_sec_per_trial=6, n_trial_per_frame=2)

    # Line noise data:
    plotter_scroll.plot_data_sliding(line_noise_matrix, events, sample_frequency, title='Line noise data',
                                     n_sec_per_trial=6, n_trial_per_frame=2)

    # Trend data:
    plotter_scroll.plot_data_sliding(trend_matrix, events, sample_frequency, title='EEG trend data',
                                     n_sec_per_trial=6, n_trial_per_frame=2)

    # Data cleaned by automatic trial rejection:
    plotter_scroll.plot_data_sliding(data_matrix_clean, events, sample_frequency, title='Clean (trial rejection) data',
                                     n_sec_per_trial=6, n_trial_per_frame=2)

    try:
        plt.draw()
        plt.show(block=True)
    except KeyboardInterrupt, ki:
        sys.exit()


