import numpy as np
import scipy.stats as stats
from sklearn.decomposition import FastICA


def ica(trial_data, n_components=10):
    trial_data = np.transpose(trial_data)

    ica = FastICA(n_components=n_components, max_iter=600)
    S_ = ica.fit_transform(trial_data)
    A_ = ica.mixing_
    ica_mean = ica.mean_

    return S_, A_, ica_mean


def blind_source_separation(ica_components, ica_mixing_system, ica_mean, n_components=10, kurtosis_threshold=10):
    for i in range(n_components):
        component = ica_components[:, i]
        kurtosis = stats.kurtosis(component)
        if kurtosis_threshold > np.fabs(kurtosis):
            ica_components[:, i] *= 0.0

    artifact_data = np.dot(ica_components, ica_mixing_system.T) + ica_mean
    return artifact_data.T