import os
import json
import constants
import scipy.io as sio


def get_json_data_from_file(user, filename):
    file_path = constants.project_path + "/database/users/" + user + "/" + filename + ".txt"
    with open(file_path) as json_data:
        data = json.load(json_data)
    return data


def get_all_trials(user):
    file_path = constants.project_path + "/database/users/" + user
    files = []

    for file in os.listdir(file_path):
        files.append(file.replace(".txt", ""))

    return files


def save_segmented_data_as_mat(data_seg, events, user, filename):
    path = constants.experiments_path_j + user + "/" + filename
    data_dict = {}
    data_dict["data_seg"] = data_seg
    data_dict["events"] = events
    sio.savemat(path, data_dict)


def save_hht_data_as_mat(frequencies, amplitudes, user, subfolder, filename):
    folder_path = constants.experiments_path_j + user + '/' + subfolder
    path = folder_path + '/' + filename
    if not os.path.exists(folder_path):
        os.makedirs(folder_path)
        fix_ownership(folder_path)

    data_dict = {}
    data_dict["frequency"] = frequencies
    data_dict["amplitude"] = amplitudes
    sio.savemat(path, data_dict)


def save_experiment_data_as_mat(data, data_name, user, filename):
    path = constants.experiments_path_j + user + "/" + filename
    data_dict = {}
    data_dict[data_name] = data
    sio.savemat(path, data_dict)


def save_mat_file_from_json_data(user, filename):
    path = constants.project_path + "/database/matlab/" + user
    create_folder(path)
    path = path + "/" + filename
    data = get_json_data_from_file(user, filename)
    sio.savemat(path, data)


def save_as_mat(data, user, subfolder, filename):
    path = constants.experiments_path + user + "/" + subfolder + "/" + filename
    json_data = {}
    json_data["imfs"] = data
    sio.savemat(path, json_data)


def save_as_settings_mat(parameters, user, subfolder, filename):
    path = constants.experiments_path + user + "/" + subfolder + "/" + filename
    json_data = {}
    json_data["parameters"] = parameters
    sio.savemat(path, json_data)


def read_mat_file_into_json_object(user, filename):
    path = constants.experiments_path_j + user + "/" + filename
    return sio.loadmat(path)


def read_mat_file_into_json_object_from_experiments(user, subfolder, filename):
    path = constants.experiments_path + user + "/" + subfolder + "/" + filename
    return sio.loadmat(path)


def save_feature_matrix_as_mat(feature_matrix, classes, user, filename):
    path = constants.project_path + "/database/matlab/" + user
    create_folder(path)
    path = path + "/" + filename
    data = {}
    data['feature_matrix'] = feature_matrix
    data['motor_classes'] = classes
    sio.savemat(path, data)


def save_json_object(path, filename, json_data):
    full_path = path + "/" + filename + ".txt"
    open(full_path, 'a').close()
    fix_ownership(full_path)
    f_write = open(full_path, 'w')
    json.dump(json_data, f_write, sort_keys=True)


# Function for changing ownership of a file or directory to gain root access
def fix_ownership(path):
    """ Change the owner of the file to SUDO_UID """
    uid = os.environ.get('SUDO_UID')
    gid = os.environ.get('SUDO_GID')
    if uid is not None:
        os.chown(path, int(uid), int(gid))


def create_folder(path):
    # Create directory for the specific user
    if not os.path.isdir(path):
        os.mkdir(path)
        # Change ownership to get root-access to the directory
        fix_ownership(path)


def get_all_trials_from_mat_file(filename):
    session_dict = read_mat_file_into_json_object_from_experiments(filename)
    session_keys = session_dict.keys()
    data_matrix_key = session_keys[5]
    events = session_dict[constants.events_matrix_name]
    data_matrix = session_dict[data_matrix_key]

    right_hand_movement_trials = {}
    left_hand_movement_trials = {}
    neutral_trials = {}

    index_right_hand_movement = 0
    index_left_hand_movement = 0
    index_neutral = 0
    n_trials = len(events[0, :])

    for i in range(n_trials):
        if events[0][i] == 'stmr':
            start = events[3][i-1]
            end = events[3][i+1]
            right_hand_movement_trials[index_right_hand_movement] = data_matrix[:, start:end]
            index_right_hand_movement += 1
        elif events[0][i] == 'stml':
            start = events[3][i-1]
            end = events[3][i+1]
            left_hand_movement_trials[index_left_hand_movement] = data_matrix[:, start:end]
            index_left_hand_movement += 1
        elif events[0][i] == 'stmn':
            start = events[3][i-1]
            end = events[3][i+1]
            neutral_trials[index_neutral] = data_matrix[:, start:end]
            index_neutral += 1

    return neutral_trials, right_hand_movement_trials, left_hand_movement_trials, n_trials





