import matplotlib.pyplot as plt
import numpy as np
from matplotlib.widgets import Slider, Button


def plot_single_data_sliding(data_array, events, fs=500.0, label='', subtract_mean=True,  n_sec_per_trial=3, n_trial_per_frame=3):

    n_sec_per_frame = n_trial_per_frame * n_sec_per_trial
    n_frames = 3*len(events)/n_sec_per_frame

    xdata = np.linspace(0, n_sec_per_frame, n_sec_per_frame*fs)
    # set up figure
    fig = plt.figure()
    ax = fig.add_subplot(111, axisbg='lightgoldenrodyellow')
    ax.autoscale(True)
    plt.subplots_adjust(left=0.03, bottom=0.09, right=0.99, top=0.96)

    # plot first data set
    frame = 0
    data = np.array(data_array[int(frame*n_sec_per_frame*fs):int((frame+1)*n_sec_per_frame*fs)])
    if subtract_mean:
        mean_data = np.mean(data)
    else:
        mean_data = 0.0
    ln, = ax.plot(xdata, data - mean_data, label=label)
    legend = ax.legend(loc='upper center', shadow=True)
    # The frame is matplotlib.patches.Rectangle instance surrounding the legend.
    legend_frame = legend.get_frame()
    legend_frame.set_facecolor('0.90')

    # Set the fontsize
    for label in legend.get_texts():
        label.set_fontsize('large')
    for label in legend.get_lines():
        label.set_linewidth(1.5)  # the legend line width

    ax.plot(0.05, 0.9*ax.get_ylim()[0], 'r+', markersize=10)
    ax.axvline(1, color='k', linestyle='--', linewidth=3)
    if n_sec_per_frame == 6:
        ax.plot(3, 0.9*ax.get_ylim()[0], 'r+', markersize=10)
        ax.axvline(4, color='k', linestyle='--', linewidth=3)
    elif n_sec_per_frame == 9:
        ax.plot(3, 0.9*ax.get_ylim()[0], 'r+', markersize=10)
        ax.axvline(4, color='k', linestyle='--', linewidth=3)
        ax.plot(6, 0.9*ax.get_ylim()[0], 'r+', markersize=10)
        ax.axvline(7, color='k', linestyle='--', linewidth=3)
    trial_num = int(frame)*n_trial_per_frame+1
    title = '(Trial ' + str(trial_num) + ', Stm = ' + str(events[trial_num-1]) + \
            ')            &           (Trial ' + str(trial_num+1) + ', Stm = ' + str(events[trial_num]) + \
            ')            &           (Trial ' + str(trial_num+2) + ', Stm = ' + str(events[trial_num+1]) + ')'
    ax.set_title(title)

    # make the slider
    axframe = plt.axes([0.13, 0.02, 0.75, 0.03], axisbg='lightgoldenrodyellow')
    sframe = Slider(axframe, 'Frame', 0, n_frames-1, valinit=0, valfmt='%d')

    # call back function
    def update(val):
        frame = np.floor(sframe.val)
        data2 = data_array[int(frame*n_sec_per_frame*fs):int((frame+1)*n_sec_per_frame*fs)]
        ln.set_xdata(xdata)
        if subtract_mean:
            mean_data2 = np.mean(data2)
        else:
            mean_data2 = 0.0
        ln.set_ydata(data2 - mean_data2)
        trial_num2 = int(frame)*n_trial_per_frame+1
        title2 = '(Trial ' + str(trial_num2) + ', Stm = ' + str(events[trial_num2-1]) + \
                 ')            &           (Trial ' + str(trial_num2+1) + ', Stm = ' + str(events[trial_num2]) + \
                 ')            &           (Trial ' + str(trial_num2+2) + ', Stm = ' + str(events[trial_num2+1]) + ')'
        ax.set_title(title2)
        ax.relim()
        ax.autoscale_view()
        plt.draw()
    # connect callback to slider
    sframe.on_changed(update)


def plot_data_sliding(data_matrix, events, fs=500.0, offset=50.0, title='', n_sec_per_trial=6, n_trial_per_frame=2, plot=plt):

    n_sec_per_frame = n_trial_per_frame * n_sec_per_trial
    n_frames = len(events)/n_trial_per_frame
    n_channels = len(data_matrix)

    xdata = np.linspace(0, n_sec_per_frame, n_sec_per_frame*fs)

    # set up figure
    fig = plot.figure()
    fig.suptitle(title)
    plot.subplots_adjust(left=0.05, bottom=0.09, right=0.98, top=0.95)
    # plot.subplots_adjust(left=0.00, bottom=0.10, right=1.01, top=0.95)
    ax = fig.add_subplot(111, axisbg='lightgoldenrodyellow')
    ax.autoscale(True)

    # plot first data set
    frame = 0
    lns = []

    for i in range(n_channels):
        data = np.array(data_matrix[i, int(frame*n_sec_per_frame*fs):int((frame+1)*n_sec_per_frame*fs)])

        if n_channels == 2:
            if i == 0:
                ln, = ax.plot(xdata, data - np.mean(data) + (i*offset), label='C3')
            elif i == 1:
                ln, = ax.plot(xdata, data - np.mean(data) + (i*offset), label='C4')
        elif n_channels == 3:
            if i == 0:
                ln, = ax.plot(xdata, data - np.mean(data) + (i*offset), label='C3')
            elif i == 1:
                ln, = ax.plot(xdata, data - np.mean(data) + (i*offset), label='C4')
            elif i == 2:
                ln, = ax.plot(xdata, data - np.mean(data) + (i*offset), label='Cz')
        else:
            ln, = ax.plot(xdata, data - np.mean(data) + (i*offset))
        lns.append(ln)

    if n_channels <= 13:
        plt.ylim(lns[0].get_ydata().min()-0.5*offset, lns[-1].get_ydata().max() + 0.5*offset)
        height = lns[-1].get_ydata().max() + 0.6*offset
    else:
        plt.ylim(lns[0].get_ydata().min()-2.0*offset, lns[-1].get_ydata().max() + 2.0*offset)
        height = lns[-1].get_ydata().max() + 2.15*offset

    if n_channels == 2 or n_channels == 3:
        legend = ax.legend(loc='upper center', shadow=True)
        # The frame is matplotlib.patches.Rectangle instance surrounding the legend.
        legend_frame = legend.get_frame()
        legend_frame.set_facecolor('0.90')

        # Set the fontsize
        for label in legend.get_texts():
            label.set_fontsize('large')

        for label in legend.get_lines():
            label.set_linewidth(1.5)  # the legend line width

    if n_sec_per_trial == 8:
        ax.axvline(2, color='k', linestyle='--', linewidth=3)

        if n_trial_per_frame == 2:
            ax.axvline(10, color='k', linestyle='--', linewidth=3)

    elif n_sec_per_trial == 6:
        ax.axvline(1, color='k', linestyle='--', linewidth=3)

        if n_sec_per_frame == 12:
            ax.axvline(7, color='k', linestyle='--', linewidth=3)
            # ax.axvline(6, color='k', linestyle='-', linewidth=4)
        elif n_sec_per_frame == 18:
            ax.axvline(7, color='k', linestyle='--', linewidth=3)
            ax.axvline(13, color='k', linestyle='--', linewidth=3)

    else:
        ax.axvline(1, color='k', linestyle='--', linewidth=3)

        if n_sec_per_frame == 6:
            ax.axvline(4, color='k', linestyle='--', linewidth=3)
        elif n_sec_per_frame == 9:
            ax.axvline(4, color='k', linestyle='--', linewidth=3)
            ax.axvline(7, color='k', linestyle='--', linewidth=3)

    trial_num = int(frame)*n_trial_per_frame+1
    if n_trial_per_frame == 1:
        title = '(Trial ' + str(trial_num) + ', Stm = ' + str(events[trial_num-1]) + ')'
        t1 = ax.text(1.1, height, title, fontsize=14)

    # CHANGE BACK
    elif n_trial_per_frame == 2:
        title1 = '(Trial ' + str(trial_num) + ', Stm = ' + str(events[trial_num-1]) + ')'
        title2 = '(Trial ' + str(trial_num+1) + ', Stm = ' + str(events[trial_num]) + ')'
        t1 = ax.text(1.1, height, title1, fontsize=14)
        t2 = ax.text(7.1, height, title2, fontsize=14)

    elif n_trial_per_frame == 3:
        title1 = '(Trial ' + str(trial_num) + ', Stm = ' + str(events[trial_num-1]) + ')'
        title2 = '(Trial ' + str(trial_num+1) + ', Stm = ' + str(events[trial_num]) + ')'
        title3 = '(Trial ' + str(trial_num+2) + ', Stm = ' + str(events[trial_num+1]) + ')'
        t1 = fig.text(1.1, height, title1, fontsize=14)
        t2 = fig.text(7.1, height, title2, fontsize=14)
        t3 = fig.text(13.1, height, title3, fontsize=14)

    # make the slider
    axframe = plot.axes([0.13, 0.02, 0.75, 0.03], axisbg='lightgoldenrodyellow')
    sframe = Slider(axframe, 'Frame', 0, n_frames-1, valinit=0, valfmt='%d')

    # call back function
    def update(val):
        frame = np.floor(sframe.val)
        for txt in ax.texts:

            txt.set_visible(False)
        for k in range(n_channels):

            data2 = data_matrix[k, int(frame*n_sec_per_frame*fs):int((frame+1)*n_sec_per_frame*fs)]
            lns[k].set_xdata(xdata)
            lns[k].set_ydata(data2 - np.mean(data2) + (k*offset))

        trial_num = int(frame)*n_trial_per_frame+1
        if n_trial_per_frame == 1:
            title21 = '(Trial ' + str(trial_num) + ', Stm = ' + str(events[trial_num-1]) + ')'
            t1 = ax.text(1.1, height, title21, fontsize=14)

        elif n_trial_per_frame == 2:
            title21 = '(Trial ' + str(trial_num) + ', Stm = ' + str(events[trial_num-1]) + ')'
            title22 = '(Trial ' + str(trial_num+1) + ', Stm = ' + str(events[trial_num]) + ')'
            t1 = ax.text(1.1, height, title21, fontsize=14)
            t2 = ax.text(7.1, height, title22, fontsize=14)

        elif n_trial_per_frame == 3:
            title21 = '(Trial ' + str(trial_num) + ', Stm = ' + str(events[trial_num-1]) + ')'
            title22 = '(Trial ' + str(trial_num+1) + ', Stm = ' + str(events[trial_num]) + ')'
            title23 = '(Trial ' + str(trial_num+2) + ', Stm = ' + str(events[trial_num+1]) + ')'
            t1 = ax.text(1.1, height, title21, fontsize=14)
            t2 = ax.text(7.1, height, title22, fontsize=14)
            t3 = ax.text(13.1, height, title23, fontsize=14)
        ax.relim()
        ax.autoscale_view()
        # plot.draw()
    # connect callback to slider
    n_trials = len(data_matrix[0]) / (n_sec_per_trial*fs)
    if n_trials > n_trial_per_frame:
        sframe.on_changed(update)


def plot_erp_sliding(data_matrix, events, fs=500.0, offset=50.0, n_sec_per_trial=1, n_trial_per_frame=3):

    n_sec_per_frame = n_trial_per_frame * n_sec_per_trial
    n_frames = len(events)/n_trial_per_frame
    n_channels = len(data_matrix)

    xdata = np.linspace(-200, n_sec_per_frame*800, n_sec_per_frame*fs)
    print np.shape(xdata)

    # set up figure
    fig = plt.figure()
    plt.subplots_adjust(left=0.03, bottom=0.09, right=0.99, top=0.96)
    ax = fig.add_subplot(111, axisbg='lightgoldenrodyellow')
    ax.autoscale(True)

    # plot first data set
    frame = 0
    lns = []

    for i in range(n_channels):
        data = np.array(data_matrix[i, int(frame*n_sec_per_frame*fs):int((frame+1)*n_sec_per_frame*fs)])

        if n_channels == 2:
            if i == 0:
                ln, = ax.plot(xdata, data - np.mean(data) + (i*offset), label='C3')
            elif i == 1:
                ln, = ax.plot(xdata, data - np.mean(data) + (i*offset), label='C4')
        else:
            if i==0 :
                print np.shape(data)
            ln, = ax.plot(xdata, data - np.mean(data) + (i*offset))
        lns.append(ln)

    if n_channels == 2:
        legend = ax.legend(loc='upper center', shadow=True)
        # The frame is matplotlib.patches.Rectangle instance surrounding the legend.
        legend_frame = legend.get_frame()
        legend_frame.set_facecolor('0.90')

        # Set the fontsize
        for label in legend.get_texts():
            label.set_fontsize('large')

        for label in legend.get_lines():
            label.set_linewidth(1.5)  # the legend line width

    trial_num = int(frame)*n_trial_per_frame+1
    if n_trial_per_frame == 1:
        ax.axvline(0, color='k', linestyle='--', linewidth=3)
        title = '(Trial ' + str(trial_num) + ', Stm = ' + str(events[trial_num-1]) + ')'

    elif n_trial_per_frame == 2:
        ax.axvline(0, color='k', linestyle='--', linewidth=3)
        ax.axvline(1000, color='k', linestyle='--', linewidth=3)
        title = '(Trial ' + str(trial_num) + ', Stm = ' + str(events[trial_num-1]) + \
                 ')            &           (Trial ' + str(trial_num+1) + ', Stm = ' + str(events[trial_num]) + ')'

    elif n_trial_per_frame == 3:
        ax.axvline(0, color='k', linestyle='--', linewidth=3)
        ax.axvline(1000, color='k', linestyle='--', linewidth=3)
        ax.axvline(2000, color='k', linestyle='--', linewidth=3)
        title = '(Trial ' + str(trial_num) + ', Stm = ' + str(events[trial_num-1]) + \
                 ')            &           (Trial ' + str(trial_num+1) + ', Stm = ' + str(events[trial_num]) + \
                 ')            &           (Trial ' + str(trial_num+2) + ', Stm = ' + str(events[trial_num+1]) + ')'
    ax.set_title(title)

    # make the slider
    axframe = plt.axes([0.13, 0.02, 0.75, 0.03], axisbg='lightgoldenrodyellow')
    sframe = Slider(axframe, 'Frame', 0, n_frames-1, valinit=0, valfmt='%d')

    # call back function
    def update(val):
        frame = np.floor(sframe.val)
        for k in range(n_channels):

            data2 = data_matrix[k, int(frame*n_sec_per_frame*fs):int((frame+1)*n_sec_per_frame*fs)]
            lns[k].set_xdata(xdata)
            lns[k].set_ydata(data2 - np.mean(data2) + (k*offset))

        trial_num2 = int(frame)*n_trial_per_frame+1
        if n_trial_per_frame == 1:
            title2 = '(Trial ' + str(trial_num2) + ', Stm = ' + str(events[trial_num2-1]) + ')'

        elif n_trial_per_frame == 2:
            title2 = '(Trial ' + str(trial_num2) + ', Stm = ' + str(events[trial_num2-1]) + \
                     ')            &           (Trial ' + str(trial_num2+1) + ', Stm = ' + str(events[trial_num2]) + ')'

        elif n_trial_per_frame == 3:
            title2 = '(Trial ' + str(trial_num2) + ', Stm = ' + str(events[trial_num2-1]) + \
                     ')            &           (Trial ' + str(trial_num2+1) + ', Stm = ' + str(events[trial_num2]) + \
                     ')            &           (Trial ' + str(trial_num2+2) + ', Stm = ' + str(events[trial_num2+1]) + ')'
        ax.set_title(title2)
        ax.relim()
        ax.autoscale_view()
        plt.draw()
    # connect callback to slider
    sframe.on_changed(update)


